#include "config.h"
#include "resoff.h"
#include "mask.h"

/*======================================================================
  Function cancel
  Input : filename of database, argument number and arguments from client
          
  Output : update database, sending code to client, and error messages
  cancel a reservation from database file
======================================================================*/
void cancel (int fd, int connfd, char **argv, uid_t uid)
{
    int desk_number;                 /* desk number */
    int db_index;                    /* database block index */
    int loc;                         /* location of a file */
    int len;                         /* number of bytes written */
    Room slot;                       /* a time block */
    sigset_t newset, oldset;         /* new and old signal sets */
    char buf[3];                     /* sending message to client */
    extern int isdaemon;
    extern pthread_mutex_t IOM;

    db_index = get_db_index (argv);  /* from arguments, get index of block */
    if ( db_index < 0 ) {            /* unreachable as it has checked */
        if ( isdaemon ) {
            D_ERR_QUIT("internal error: no db_index");
        }
        else {
            ERR_QUIT("internal error: no db_index");
        }
    }

    loc = db_index * sizeof(slot);   /* calculate location of a block */

    /*fd = open (fn, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
    if ( fd < 0 ) {
        openerrmsg();
        remove_lock(lf);
        exit(ERROR);
        }*/

    SIGFILL(newset);
    BLOCK(newset, oldset);
    pthread_mutex_lock(&IOM);
    CHECK (lseek (fd, loc, SEEK_SET));
    CHECK (len = read (fd, &slot, sizeof(slot))); /* read a block to update */
    pthread_mutex_unlock(&IOM);
    UNBLOCK(newset, oldset);

    desk_number = get_desk_number(slot, argv, uid);

    /* update a block to be empty */
    if ( desk_number < 4 ) {
        slot.count--;
	slot.empty[desk_number] = TRUE;
	BLOCK(newset, oldset);
        pthread_mutex_lock(&IOM);
	CHECK (lseek (fd, loc, SEEK_SET));
	CHECK (len = write (fd, &slot, sizeof(slot)));
        pthread_mutex_unlock(&IOM);
	UNBLOCK(newset, oldset);
        snprintf(buf, sizeof(buf), "0\6");
        if ( (write(connfd, buf, strlen(buf))) != strlen(buf) ) {
            if ( isdaemon ) {
                D_ERR_QUIT ("write in cancel");
            }
            else {
                ERR_QUIT ("write in cancel");
            }
        }
        
    }
    else {
        switch ( desk_number ) {
            case 222:
                snprintf(buf, sizeof(buf), "2\6");
                break;
            case 333:
                snprintf(buf, sizeof(buf), "3\6");
                break;
            case 444:
                snprintf(buf, sizeof(buf), "4\6");
                break;
            default:
                if ( isdaemon ) {
                    D_ERR_QUIT ("internal error: invalid desk number return");
                }
                else {
                    ERR_QUIT ("internal error: invalid desk number return");
                }
                break;
        }
        if ( (write(connfd, buf, strlen(buf))) != strlen(buf) ) {
            if ( isdaemon ) {
                D_ERR_QUIT ("write in cancel");
            }
            else {
                ERR_QUIT ("write in cancel");
            }
        }
        
    }
     
    CHECK (close(fd));
}

/*======================================================================
  Function get_desk_number
  Input : a time block, arguments of clinet commands, and user id
  Output : return matched desk number or error code number
           error code 222 : no user name
                      333 : no reservation matching
                      444 : not allowed uid to cancel
                      999 : internal error
  find a user matched desk or check validity of input cancel options
======================================================================*/
int get_desk_number(Room slot, char **argv, uid_t uid) {

    int i;                      /* loop index */
    uid_t euid;                 /* effective user id */
    uid_t target_uid;           /* target user id */
    struct passwd *puid;        /* passwd for user name and uid */

    euid = geteuid();
    puid = getpwnam(argv[4]);
    if ( puid == NULL ) {       /* there is no such user name */
        return(222);
    }
    target_uid = puid->pw_uid;
    /* if user id is owner of database or user name is the user's */
    /* then search given user's desk                              */
    if ( uid == euid || target_uid == uid ) {
        for (i=0; i<4; i++) {
            if ( !slot.empty[i] ) {
                if ( target_uid == slot.desk[i].uid ) {
                    return (i);
                }
            }
        }
        return (333);           /* no reservation matching */
    }
    else {         /* uid is not allowed to cancel other user's reservation */
        return (444);
    }
    
    return (999);                     /* large enough not to match any desk */

}


