#include "config.h"
#include "resoff.h"

/*======================================================================
  Function error_quit
  Input : string code
  Output : error message
  print error message and exit
======================================================================*/
void error_quit (char code)
{
    fprintf(stderr, "ERROR: %c ", code);

    switch(code) {
        case '0':             /* unreachable only for debugging */
            fprintf(stderr, "There is no error\n");
            break;
        case '1':
            fprintf(stderr, "The first character transmitted is not "
                    "recognized as a valid command.\n");
            break;
        case '2':
            fprintf(stderr, "The <userid> or <username> is not valid on "
                    "the client or server system.\n");
            break;
        case '3':
            fprintf(stderr, "There is no reservation matching the values "
                    "given by the parameters.\n");
            break;
        case '4':
            fprintf(stderr, "The user does not have permission to delete "
                    "the specified reservation on the server system.\n");
            break;
        case '5':
            fprintf(stderr, "The message from the client violates the "
                    "protocol syntax.\n");
            break;
        case '6':
            fprintf(stderr, "The client was unable to find the address "
                    "of the server.\n");
            break;
        case '7':
            fprintf(stderr, "The client was unable to connect to the "
                    "server.\n");
            break;
        case '8':
            fprintf(stderr, "The client lost connection to the server "
                    "before the transaction was completed.\n");
            break;
        case '9':
            fprintf(stderr, "A system error on the server side prevented "
                    "completion of the transaction.\n");
            break;
        case 'A':
            fprintf(stderr, "The client timed out before the server "
                    "completed the transaction.\n");
            break;
        case 'B':
            fprintf(stderr, "The response from the server violates "
                    "the protocol syntax.\n");
            break;
        case 'C':
            fprintf(stderr, "The output from the server for a l command "
                    "exceeds the maximum line count of 5000 lines.\n");
            break;
        case 'D':
            fprintf(stderr, "The command-line input to the client "
                    "is ill-formed.\n");
            break;
        case 'E':
            fprintf(stderr, "Other errors, without their own codes.\n");
            break;
        case 'F':
            fprintf(stderr, "The reservation violates the rule about "
                    "one person booking more than 3 hours in a day.\n");
            break;
        case 'G':
            fprintf(stderr, "The command-line input to the client "
                    "is ill-formed.\n");
            break;
        case 'H':
            fprintf(stderr, "The reservation cannot be made because "
                    "there is no empty desk at the requested time.\n");
            break;
        case 'I':
            fprintf(stderr, "The reservation cannot be made because "
                    "another TA for the same course has booked a desk "
                    "at that time.\n");
            break;
        default:
            fprintf(stderr, "Not defined error code\n");
            break;
    }
    
    exit (ERROR);
}

/*======================================================================
  Function print_usage
  Input : none
  Output : command usage message
  print out a usage
======================================================================*/
void print_usage () {

    fprintf (stderr, "resoffc, version 1.0\n\n"
	     "usage: resoffc -r day start_time course_number\n"
	     "  or:  resoffc -d day start_time [username]\n"
	     "  or:  resoffc -l\n\n"
	     "  -r\treserve a desk\n"
	     "  -d\tcancel the reservation\n"
	     "  -l\tlist out the reservations\n");

}
