#include "config.h"
#include "resoff.h"
#include "mask.h"

/*======================================================================
  Function list
  Input : database file descriptor and socket descriptor
  Output : sending list of reserved desks
           heading, day, time, user name, course name
  sending the contents of the database
======================================================================*/
void list (int fd, int connfd) {

    int i, j;                  /* loop index */
    extern char tbuf[7][1200]; /* buffer for printing day data  */
                               /* becase actual data is not     */
                               /* orderd in MTWRFSX but MWFTRSX */
    char daybuf[7][1200];      /* converted buffer from tbuf */
    char sbuf[1200];           /* sending buffer to client */
    char str[128];             /* string for user name and course name */
    char aline[80];            /* a line of string */
    char *ptr;                 /* string pointer for reading a line */
    int count;                 /* counter of lines in a buffer */
    int len;                   /* number of bytes read */
    int n;                     /* number of char in a line */
    int loc;                   /* block location */
    int day;                   /* numbered day */
    Room slot;                 /* a time block */
    struct passwd *user;       /* passwd struct for user name */
    sigset_t newset, oldset;   /* new and old signal sets */
    extern int isdaemon;
    extern pthread_mutex_t IOM;

    SIGFILL(newset);
    BLOCK(newset, oldset);
    pthread_mutex_lock(&IOM);

    print_head(connfd);        /* printing header of reservation table */

    for (i=0; i<69; i++) {
        day = print_day(i);     /* print a day and return day value */
	loc = i * sizeof(slot); /* block location */
	CHECK (lseek(fd, loc, SEEK_SET));
        CHECK (len = read (fd, &slot, sizeof(slot)));
	if ( slot.count > 0 ) {
	    time_print(day, i);
	    for (j=0; j<4; j++) {
	        if ( !slot.empty[j] ) {
		    user = getpwuid(slot.desk[j].uid); /* get user name */
		    sprintf (str, " %s %s", user->pw_name,
			    slot.desk[j].class);
		    strcat (tbuf[day], str);
		}
	    }
	}
    }

    sprintf(daybuf[0], "\n%s\n", tbuf[0]);             /* Monday    */
    sprintf(daybuf[1], "\n%s\n", tbuf[3]);             /* Tuesday   */
    sprintf(daybuf[2], "\n%s\n", tbuf[1]);             /* Wednesday */
    sprintf(daybuf[3], "\n%s\n", tbuf[4]);             /* Thursday  */
    sprintf(daybuf[4], "\n%s\n", tbuf[2]);             /* Friday    */
    sprintf(daybuf[5], "\n%s\n", tbuf[5]);             /* Saturday  */
    sprintf(daybuf[6], "\n%s\n", tbuf[6]);             /* Sunday    */

    pthread_mutex_unlock(&IOM);
    UNBLOCK(newset, oldset);
    
    for (i=0; i<7; i++) {
        count = 0;
        len = strlen(daybuf[i]);
        ptr = &daybuf[i][0];

        for (j=0; j<12; j++) {
            if ( count >= len ) break;
            n = cpyline(ptr, aline, 80);
            aline[n] = 0;
            snprintf(sbuf, n+1, "%s", aline);
            if ( (write(connfd, sbuf, n+1)) != (n+1) ) {
                if ( isdaemon ) {
                    D_ERR_QUIT ("write in list");
                }
                else {
                    ERR_QUIT ("write in list");
                }
            }
            count += n;
            ptr = &daybuf[i][++count];                     /* next line */
        }
        
    }
    
}

/*======================================================================
  Function print_head
  Input : socket descriptor to send
  Output : sending header printing
  sending a header printing to client
======================================================================*/
void print_head(int connfd)
{
    char sbuf[80];
    extern int isdaemon;

    snprintf(sbuf, 63, "-------------------------------"
            "-------------------------------");
    if ( (write(connfd, sbuf, 63)) != 63 ) {
        if ( isdaemon ) {
            D_ERR_QUIT ("write in list");
        }
        else {
            ERR_QUIT ("write in list");
        }
    }
    
    snprintf(sbuf, 40, "                     Office Reservation");
    if ( (write(connfd, sbuf, 40)) != 40 ) {
        if ( isdaemon ) {
            D_ERR_QUIT ("write in list");
        }
        else {
            ERR_QUIT ("write in list");
        }
    }
    
    snprintf(sbuf, 63, "-------------------------------"
             "-------------------------------");
    if ( (write(connfd, sbuf, 63)) != 63 ) {
        if ( isdaemon ) {
            D_ERR_QUIT ("write in list");
        }
        else {
            ERR_QUIT ("write in list");
        }
    }
}

/*======================================================================
  Function print_day
  Input : index of a time block in the database file
          MWF : 11 time blocks a day  TRSX : 9 time blocks a day
  Output : print days on correspond buffer array
           return a numbered value of a day ( eg. Monday 0 Tuesday 3 )
  print to buffer a day name and return numbered value of a day
======================================================================*/
int print_day(int i)
{
    int day;                             /* numbered value of a day */
    extern char tbuf[7][1200];           /* temporary buffer to print */

    switch(i) {
    case 0:                              /* starting point of Monday */
        sprintf(tbuf[0], "Monday  ");
	return (0);
        break;
    case 11:                             /* starting point of Wednesday */
        sprintf(tbuf[1], "Wednesday");
	return (1);
        break;
    case 22:
        sprintf(tbuf[2], "Friday  ");
	return (2);
        break;
    case 33:                             /* starting point of Tuesday */
        sprintf(tbuf[3], "Tuesday ");
	return (3);
        break;
    case 42:
        sprintf(tbuf[4], "Thursday");
	return (4);
	break;
    case 51:
        sprintf(tbuf[5], "Saturday");
	return (5);
        break;
    case 60:
        sprintf(tbuf[6], "Sunday ");
	return (6);
        break;
    default:
      day = get_day(i);           /* calculate corresponded day value */
	return (day);
        break;
    }

    return(ERROR);

}

/*======================================================================
  Function get_day
  Input : index of a time block in the database file
  Output : return a numbered value of a day ( eg. Monday 0 Tuesday 3 )
  return a numbered value corresponded block index of the database file
======================================================================*/
int get_day(int i)
{
    if ( i < 11 ) { return (0); }
    else if ( i < 22 ) { return (1); }
    else if ( i < 33 ) { return (2); }
    else if ( i < 42 ) { return (3); }
    else if ( i < 51 ) { return (4); }
    else if ( i < 60 ) { return (5); }
    else { return (6); }
    
    return (ERROR);

}

/*======================================================================
  Function time_print
  Input : day value, index of a time block in the database file
  Output : string buffer filled with corresponded time
  from day and block index, print a time string corresponding index
======================================================================*/
int time_print(int day, int i)
{    
    int index;                  /* index of time of each day */
    char str[128];              /* temporary index to save time print */
    extern char tbuf[7][1200];  /* temporary buffer to print each day */
    char mwf_day[11][6] = {     /* Monday, Wednesday, Friday times */
        "08:00", "09:05", "10:10", "11:15", "12:20",
	"13:25", "14:30", "15:35", "17:15", "18:45", "20:15" };
    char trsx_day[9][6] = {     /* the other days times */
        "08:00", "09:30", "11:00", "12:30",
	"14:00", "15:35", "17:15", "18:45", "20:15" };

    switch (day) {
    case 0:
    case 1:
    case 2:
        index = i % 11;         /* MWF has 11 time blocks */
	sprintf (str, "\n%s ", mwf_day[index]);
	strcat (tbuf[day], str);
	break;
    case 3:
    case 4:
    case 5:
    case 6:
        index = ( i - 33 ) % 9; /* TRSX has 9 time blocks after MWF blocks */
	sprintf(str, "\n%s ", trsx_day[index]);
	strcat (tbuf[day], str);
	break;
    default:
        break;
    }

    return (0);

}
