#include "config.h"
#include "resoff.h"
#include "mask.h"

/*======================================================================
  Function reserve
  Input : file description, socket descriptor, arguments from client
          and user id
  Output : time block data saved in database file and error code sending
  reserving a desk on a time slot
======================================================================*/
void reserve (int fd, int connfd, char **argv, uid_t uid)
{
    int i;
    int db_index;                  /* index of a time block */
    int loc;                       /* location of a random access file */
    int len;                       /* length of a string */
    Room slot;                     /* a time block */
    sigset_t newset, oldset;       /* old and new signal masks */
    char buf[4];                   /* sending message to client */
    char err;                      /* error code to return */
    extern int isdaemon;
    extern pthread_mutex_t IOM;
    
    course_check (argv[4]);        /* error check of course name */

    db_index = get_db_index(argv); /* get corresponding index in database */
    if ( db_index < 0 ) {          /* unreachable as it has checked */
        if ( isdaemon ) {
            D_ERR_QUIT("internal error: no db_index");
        }
        else {
            ERR_QUIT("internal error: no db_index");
        }
    }
    loc = db_index * sizeof(slot); /* location of corresponding block */
        
    SIGFILL(newset);              /* setup full signal mask */
    BLOCK(newset, oldset);        /* block system interrupt */
    pthread_mutex_lock (&IOM);    /* I/O mutex for safe I/O */
    
    CHECK (lseek (fd, loc, SEEK_SET));
    CHECK (len = read (fd, &slot, sizeof(slot)));

    pthread_mutex_unlock (&IOM);  /* unlock I/O mutex */
    UNBLOCK(newset, oldset);      /* unblock system interrupt */

    /* check empty desk and validity of input, and update data */
    if ( slot.count > -1 && slot.count < 4 ) {  /* count is 0, 1, 2, 3 */
        slot.count++;
	make_upper(argv[4]);                    /* upper case of course name */

        /* input validity check e.g. more than 3 times a day */
	if ( (err = validity(db_index, slot, argv[4], fd, uid)) ) {
            snprintf(buf, sizeof(buf), "0%c\6", err);
            if ( (write(connfd, buf, strlen(buf))) != strlen(buf) ) {
                if ( isdaemon ) {
                    D_ERR_QUIT ("write in reserve");
                }
                else {
                    ERR_QUIT ("write in reserve");
                }
            }
            return;
        }
        
        for (i=0; i<4; i++) {
	    if ( slot.empty[i] ) {              /* if a desk is empty */
	        slot.empty[i] = FALSE;
	        slot.desk[i].uid = uid;
		make_upper(argv[4]);
		strcpy(slot.desk[i].class, argv[4]);
		BLOCK(newset, oldset);
                pthread_mutex_lock(&IOM);
		CHECK (lseek (fd, loc, SEEK_SET));
		CHECK (len = write (fd, &slot, sizeof(slot)));
                pthread_mutex_unlock(&IOM);
		/*CHECK (close(fd));*/
		UNBLOCK(newset, oldset);
                snprintf(buf, sizeof(buf), "%d0\6", i+1);
                if ( (write(connfd, buf, strlen(buf))) != strlen(buf) ) {
                    if ( isdaemon ) {
                        D_ERR_QUIT ("write in reserve");
                    }
                    else {
                        ERR_QUIT ("write in reserve");
                    }
                }
                return;
	    }
	}
    }
    else if ( slot.count == 4 ) {                /* desks are full */
        snprintf(buf, sizeof(buf), "0H\6");
        if ( (write(connfd, buf, strlen(buf))) != strlen(buf) ) {
            if ( isdaemon ) {
                D_ERR_QUIT ("write in reserve");
            }
            else {
                ERR_QUIT ("write in reserve");
            }
        }
    }
    else {	                                 /* unreachable count number */
	/*CHECK (close(fd));*/
        if ( isdaemon ) {
            D_ERR_QUIT("Slot count error in reserve.\n");
        }
        else {
            ERR_QUIT("Slot count error in reserve.\n");
        }
    }
    
}
