/*----------------------------------------------------------------------
  Header file : resoff.h
  Declarations of external functions and definitions of constants.
----------------------------------------------------------------------*/

#ifndef RESOFF_H

#define RESOFF_H
#define CHECK(CALL)\
 if ((CALL) < 0) {\
    perror (#CALL);\
    _exit (-1);\
 }

#define ERR_QUIT(MSG) {\
 perror (MSG);\
 _exit (-1);\
 }

#define ERR_QUIT_2(MSG, ERRNO) {\
 fprintf (stderr, "%s: %s\n", (MSG), strerror(ERRNO));\
 _exit (-1);\
 }

/* daemon error quit macro */
#define D_ERR_QUIT(MSG) {\
 syslog(LOG_ERR, "%s", (MSG));\
 _exit (-1);\
 }

#define D_ERR_QUIT_2(MSG, ERRNO) {\
 syslog (LOG_ERR, "%s: %s", (MSG), strerror(ERRNO));\
 _exit (-1);\
 }

#define NORMAL 0
#define ERROR -1
#define TRUE   1
#define FALSE  0
#define NONFD -2              /* no file descriptor - initial state */

#define SERV_PORT 11429       /* default port number */
#define MAXCONN 5             /* maximum connection and thread number */
#define LISTENQ MAXCONN       /* maximum number of connection */
#define SA struct sockaddr    /* simplifying definition */
#define MAXLINE 1024          /* maximum characters per line */
#define MAXWAIT 30            /* maximum timeout seconds */
#define MAXFD 64              /* maximum number of file descriptor */

#define MAXTHREAD MAXCONN     /* maximum threads in the thread pool */

#define _RESERVE 1
#define _CANCEL  2
#define _LIST    3
#define _ERR_RESERVE    11
#define _ERR_CANCEL     12
#define _ERR_SYNTAX     20
#define _ERR_RSV_SYNTAX 21
#define _ERR_CNL_SYNTAX 22
#define _ERR_RSV_UID    31
#define _ERR_CNL_UID    32

typedef void Sigfunc(int);

typedef struct {       /* typedef for a desk */
  uid_t uid;
  char class[8];       /* class name */
} Desks;
typedef struct {       /* typedef for a time block */
  int count;           /* number of empty desk */
  int empty[4];        /* Boolean of empty desk */
  Desks desk[4];       /* 4 desks */
} Room;

char tbuf[7][1200];     /* buffer for printing proper order of days */
char filename[256];     /* database file name */
char lockfile[256];     /* lock file name */
int  gfd;               /* global socket descriptor */
int  isdaemon;          /* is daemon program? - TRUE or FALSE */
pthread_mutex_t M;      /* mutex variable */
pthread_mutex_t IOM;    /* I/O mutex variable */
pthread_cond_t AC;      /* condition variable - wait a client request */
pthread_cond_t EC;      /* condition variable - wait a thread available */

extern ssize_t readline(int fd, void *vptr, size_t maxlen);
extern ssize_t readword(char *lptr, void *vptr, size_t maxlen);
extern ssize_t cpyline(char *lptr, void *vptr, size_t maxlen);
extern ssize_t Readline(int fd, void *ptr, size_t maxlen);
extern ssize_t writen(int fd, const void *vptr, size_t n);
extern void Writen (int fd, void *ptr, size_t nbytes);
int daemon_init(const char *);
Sigfunc *Signal (int, Sigfunc *);
Sigfunc *signal (int, Sigfunc *);
void req_reserve (int, char **);
int  rsv_msg_chk (char *);
void rsv_result(char *);
void req_cancel (int, int, char **);
int  cancel_msg_chk (char *);
void cancel_result(char *);
void error_quit (char);
void req_list (int);
int  client_cmdchk (int, char **);
int  is_rsv_cmd (char **);
int  course_check(char *);
int  get_db_index(char **);
int  is_cancel_cmd (char **);
int  is_all_digit (char *);
void print_usage ();
void openerrmsg();
void d_openerrmsg();
void socketerrmsg();
void d_socketerrmsg();
void binderrmsg();
void d_binderrmsg();
void connerrmsg();
void *resoffserve(void *);
int  server_cmdchk(char *, int *, char **, uid_t *);
void reserve(int, int, char **, uid_t);
char validity(int, Room, char *, int, uid_t);
char check_day_reserve(int, int, uid_t);
void cancel(int, int, char **, uid_t);
int  get_desk_number(Room, char **, uid_t);
void list(int, int);
void print_head(int);
int  print_day(int);
int  get_day(int);
int  time_print(int, int);
void sig_chld(int);
void sig_alrm(int);
void sig_int(int);
void make_upper(char *);
void remove_lock(char *);
int  initdb(int);
#endif
