/***********************************************************************
  Server main function of Assignment #5 of Advanced Unix Programming.
  Input : string queries from 'resoffc' clients through TCP/IP
          "-d" as an optional main function parameter for daemon
  Output : list of current reserved desk database through TCP/IP.
  Assumption : Course number does not check.
               Total reserved blocks by a user does not check.
               Maximum number of listen queue is the same as
               the maximum number of thread.
  Author : Jungkee (Jake) Kim
  Date : November 13, 2000 (Due Date)
************************************************************************/
#include "config.h"
#include "resoff.h"
#include "mask.h"

int main(int argc, char **argv)
{
    char *envpath;                    /* environment path name */
    char *envport;                    /* environment port number */
    char buf[30];                     /* temporary buffer */
    struct passwd *home;              /* passwd struct for home directory */
    int lfd;                          /* lock file descriptor */
    int fd;                           /* file description of database file */
    int listenfd;
    int connfd;
    int i;
    int n;                            /* return value of daemon init */
    struct sockaddr_in cliaddr;       /* client socketaddr */
    struct sockaddr_in servaddr;      /* server socketaddr */
    socklen_t clilen;                 /* client value result */
    int result;                       /* result of thread creation */
    pthread_t tid;                    /* thread id */
    pthread_attr_t attr;              /* thread attributes */
    sigset_t newset, oldset;          /* new and old signal sets */
    extern char filename[256];        /* database file name */
    extern char lockfile[256];        /* lock file name */
    extern int gfd;                   /* global socket descriptor */
    extern int isdaemon;              /* is daemon program */
    extern pthread_mutex_t M;         /* mutex variable */
    extern pthread_cond_t AC;         /* wait a client request */
    extern pthread_cond_t EC;         /* wait a thread available */

    if ( argc > 2 ) {
        fprintf(stderr, "invalid option error\n");
        exit(ERROR);
    }
    
    SIGFILL(newset);                  /* setup full signals to mask */
    BLOCK(newset, oldset);            /* block all signals */

    if ( argc == 2 ) {
        if ( strlen(argv[1]) == 2 &&
             argv[1][0] == '-' && argv [1][1] == 'd' ) {
            sprintf(buf, "jungkkim-%s", argv[0]);
            n = daemon_init(buf);
            if ( n < 0 ) {
                fprintf(stderr, "deamon initial error");
                exit(ERROR);
            }
            isdaemon = TRUE;
        }
        else {
            fprintf(stderr, "invalid option error\n");
            exit(ERROR);
        }
    }
    
    envpath = getenv ("RESOFFDIR");   /* get environment path */
    if ( envpath == NULL ) {  /* module to get database file name from home */
        home = getpwuid(geteuid());
	strcpy(filename, home->pw_dir);
	strcat(filename, "/resoff.db");
	strcpy(lockfile, home->pw_dir);   /* module to get lock file name */
	strcat(lockfile, "/resoff.lock");
    }
    else {                    /* module to get database file name from env */
        strcpy(filename, envpath);
	strcat(filename, "/resoff.db");
	strcpy(lockfile, envpath);        /* module to get lock file name */
	strcat(lockfile, "/resoff.lock");
    }
    
    lfd = open (lockfile, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
    
    if ( lfd < 0 ) {
        if ( isdaemon ) {
            D_ERR_QUIT("other server is running or lock is not deleted "
                       "after termination" );
        }
        else {
            ERR_QUIT("Other server is running or lock is not deleted "
                     "after termination.\n");
        }
    }
    
    fd = open (filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);

    if ( fd < 0 ) {
        openerrmsg();
        remove_lock(lockfile);
        exit(ERROR);
    }
    
    /* unblock SIGINT */
    if ( sigdelset(&newset, SIGINT) ) {
        ERR_QUIT("sigdelset error");
    }
    if ( sigprocmask (SIG_UNBLOCK, &newset, NULL) ) {
        ERR_QUIT("sigprocmask unblock error");
    }    
    Signal(SIGINT, sig_int);               /* set  SIGINT signal handler */
    
    initdb(fd);
    close(fd);
    
    if ( (listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        socketerrmsg();
        exit(ERROR);
    }
    
    memset (&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr = htonl(INADDR_ANY);

    envport = getenv ("RESOFFPORT");       /* get environment port number */
    if ( envport == NULL ) {
        servaddr.sin_port = htons(SERV_PORT);
    }
    else {              /* check the validity of port number: all number? */
        if ( is_all_digit(envport) ) {
            servaddr.sin_port = htons(atoi(envport));
        }
        else {
            if ( isdaemon ) {
                D_ERR_QUIT("Illigal form of  environment RESOFFPORT");
            }
            else {
                ERR_QUIT("Illigal form of  environment RESOFFPORT\n");
            }
        }
    }
    
    if ( (bind (listenfd, (SA *) &servaddr, sizeof (servaddr))) < 0 ) {
        if ( isdaemon )
            d_binderrmsg();
        else
            binderrmsg();
        exit(ERROR);
    }

    if ( (listen (listenfd, LISTENQ)) < 0 ) {
        if ( isdaemon ) {
            D_ERR_QUIT("listen error");
        }
        else {
            ERR_QUIT("listen error\n");
        }
    }
    
    pthread_mutex_init(&M, NULL);            /* initialize mutex */
    pthread_cond_init(&AC, NULL);            /* initialize condition */
    
    pthread_attr_init(&attr);                /* detached attribute */
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

    gfd = NONFD;                             /* global fd initialzation */
    
    for (i=0; i<MAXTHREAD; i++) {            /* creating thread pool */ 
        result = pthread_create(&tid, NULL, resoffserve, NULL);
        if ( result ) {
            if ( isdaemon ) {
                D_ERR_QUIT_2 ("thread creation", result);
            }
            else {
                ERR_QUIT_2 ("thread creation", result);
            }
        }
    }
    
    for ( ; ; ) {
        clilen = sizeof(cliaddr);
        if ( (connfd = accept(listenfd, (SA *) &cliaddr, &clilen)) < 0) {
            if (errno == EINTR)
                continue;
            else {
                if ( isdaemon ) {
                    D_ERR_QUIT ("accept");
                }
                else {
                    ERR_QUIT ("accept");
                }
            }
        }
      
        pthread_mutex_lock(&M);
        while ( gfd > -1 )              /* if no thread is available */
            pthread_cond_wait(&EC, &M);
        gfd = connfd;
        pthread_cond_signal(&AC);     /* wake up a thread waiting a request */
        pthread_mutex_unlock(&M);

    }

    exit(0);
}
