/*
 File: resofflib.c
 library functions for resoff through TCP/IP and deamon
 no commented functions from the text, "Unix network programming"
*/

#include "config.h"
#include "resoff.h"

static ssize_t
my_read(int fd, char *ptr)
{
   static int   read_cnt = 0;
   static char *read_ptr;
   static char  read_buf[MAXLINE];

   if (read_cnt <= 0) {
again:
      if ( (read_cnt = read(fd, read_buf, sizeof(read_buf))) < 0) {
         if (errno == EINTR) goto again;
         return(-1);
      } else if (read_cnt == 0) return(0);
      read_ptr = read_buf;
   }
   read_cnt--;
   *ptr = *read_ptr++;
   return (1);
}

ssize_t
readline(int fd, void *vptr, size_t maxlen)
{
   int      n, rc;
   char   c, *ptr;

   ptr = vptr;
   for (n = 1; n < maxlen; n++) {
      if ( (rc = my_read(fd, &c)) == 1) {
         *ptr++ = c;
         if (c == '\0' || c == '\6' ) break; /* end character */
      } else if (rc == 0) {
         if (n == 1) return(0); /* EOF, no data read */
         else break;            /* EOF, some data was read */
      } else return(0);        /* error, errno set by read() */
      /* what happens if we return -1 ? */
   }

   *ptr = 0;   /* null terminate like fgets() */
   return(n);
}


/*======================================================================
  Function readword
  Input : character pointers for source and a word, maximum length
  Output : pointer for a word and next point word for source
  read a word from given source string
======================================================================*/
ssize_t
readword(char *lptr, void *vptr, size_t maxlen)
{
   int  n;
   char *ptrl, *ptrv;

   ptrl = lptr;                            /* copy pointer of source */
   ptrv = vptr;                            /* copy pointer of word */
   for (n = 0; n < maxlen; n++) {          /* copy characters */
       if ( *ptrl == ' ' || *ptrl == '\0' ) break;
       *ptrv++ = *ptrl++;
   }
   *ptrv = 0;                              /* set string end */ 
   return (n);                             /* return length */
}

/*======================================================================
  Function cpyline
  Input : character pointers for source and a line, maximum length
  Output : pointer for a line and next point line for source
  read a line from given source string
======================================================================*/
ssize_t cpyline(char *lptr, void *vptr, size_t maxlen) 
{
    int n;
    char *ptrl, *ptrv;
    
    ptrl = lptr;                            /* copy pointer of source */
    ptrv = vptr;                            /* copy pointer of line */
    for (n = 0; n < maxlen; n++) {          /* copy characters */
        if ( *ptrl == '\n' || *ptrl == '\0' ) break;
        *ptrv++ = *ptrl++;
    }
    *ptrv = 0;                              /* set string end */
    return (n);                             /* return length */
}
    
ssize_t
Readline(int fd, void *ptr, size_t maxlen)
{
   ssize_t      n;
   if ( (n = readline(fd, ptr, maxlen)) < 0)
      ERR_QUIT ("readline error");
   return(n);
}

ssize_t 
writen(int fd, const void *vptr, size_t n)
/* Write "n" bytes to a descriptor. */
{
   size_t      nleft;
   ssize_t     nwritten;
   const char *ptr;

   ptr = vptr;
   nleft = n;
   while (nleft > 0) {
      if ( (nwritten = write(fd, ptr, nleft)) <= 0) {
         if (errno == EINTR) nwritten = 0;  /* and call write() again */
         else return(-1);        /* error */
      }
      nleft -= nwritten;
      ptr   += nwritten;
   }
   return(n);
}

void
Writen (int fd, void *ptr, size_t nbytes)
{
   if (writen(fd, ptr, nbytes) != nbytes)
      ERR_QUIT ("writen error");
}

/*======================================================================
  Function daemon_init
  Input : ident string
  Output : integer 0(normal) or -1(error)
  initialize the calling program as a daemon
======================================================================*/
int daemon_init(const char *ident) 
{
    pid_t pid;
    int i;
    
    if ( (pid = fork() ) < 0 )
        return (-1);
    else if (pid != 0)
        exit(0);
    
    setsid();
    
    chdir("/");
    
    umask(0);

    for (i=0; i<MAXFD; i++)
        close(i);

    openlog(ident, LOG_PID, LOG_USER);

    return(0);
}
