#include "config.h"
#include "resoff.h"

/*======================================================================
  Function resoffserve
  Input : none exept global variables : connection file descriptor and
          other global variables - isdaemon, AC
  Output : return string code to client
  server interpretation of client code and execute corresponding
  functions
======================================================================*/
void *resoffserve(void *vptr) 
{
    int fd;                           /* file descriptor */
    int option;                       /* kind of options */
    int argc;                         /* pseudo number of arguments */
    char *argv[5];                    /* pseudo arguments of main function */
    uid_t uid;                        /* user id */
    char buf[4];                      /* sending error message to client */
    char recvline[30];                /* received command */
    int i, n;
    int connfd;
    extern int isdaemon;              /* is daemon program */
    extern pthread_mutex_t M;         /* mutex variable */ 
    extern pthread_cond_t AC;         /* wait a client request */
    extern pthread_cond_t EC;         /* wait a thread available */

    for (;;) {
        pthread_mutex_lock(&M);
        while ( gfd < 0 )             /* if no connection - wait */
            pthread_cond_wait(&AC, &M);
        connfd = gfd;                 /* get connect descriptor from global */
        gfd = NONFD;                  /* initialize global descriptor */
        pthread_mutex_unlock(&M);
        
        if ( (n = read(connfd, recvline, 30)) != 0 ) {
            if ( n == -1 ) {            /* for debug */
                if ( isdaemon ) {
                    syslog(LOG_ERR, strerror(errno));
                    exit (ERROR);
                }
                else {
                    perror("resoffserve - read");
                    exit(-1);
                }
            }
            /* check command */
            option = server_cmdchk(recvline, &argc, argv, &uid);
            if ( !option ) {                  /* if no proper option */
                snprintf(buf, 3, "1\6");
                if ( (write(connfd, buf, strlen(buf))) != strlen(buf) ) {
                    if ( isdaemon )
                        syslog(LOG_ERR, "write in resoffserve");
                    else
                        perror ("write in resoffserve");
                }
            }
            else {                            /* got option */
                fd = open (filename, O_RDWR, S_IRUSR | S_IWUSR);
                if ( fd < 0 ) {
                    openerrmsg();
                    remove_lock(lockfile);
                    exit(ERROR);
                }
                switch( option ) {
                    case _RESERVE:
                        reserve(fd, connfd, argv, uid);
                        break;
                    case _CANCEL:
                        cancel(fd, connfd, argv, uid);
                        break;
                    case _LIST:
                        list(fd, connfd);
                        break;
                    case _ERR_RESERVE:
                    case _ERR_CANCEL:
                    case _ERR_SYNTAX:
                    case _ERR_CNL_SYNTAX:
                        snprintf(buf, 3, "5\6");
                        if ((write(connfd, buf, strlen(buf))) != strlen(buf)) {
                            if ( isdaemon ) {
                                D_ERR_QUIT ("write in syntax error");
                            }
                            else {
                                ERR_QUIT ("write in syntax error");
                            }
                        }
                        break;
                    case _ERR_RSV_SYNTAX:
                        snprintf(buf, sizeof(buf), "05\6");
                        if ((write(connfd, buf, strlen(buf))) != strlen(buf)) {
                            if ( isdaemon ) {
                                D_ERR_QUIT ("write in rsv syntax error");
                            }
                            else {
                                ERR_QUIT ("write in rsv syntax error");
                            }
                        }
                        break;
                    case _ERR_RSV_UID:
                        snprintf(buf, sizeof(buf), "02\6");
                        if ((write(connfd, buf, strlen(buf))) != strlen(buf)) {
                            if ( isdaemon ) {
                                D_ERR_QUIT ("write in rsv uid error");
                            }
                            else {
                                ERR_QUIT ("write in rsv uid error");
                            }
                        }
                        break;
                    case _ERR_CNL_UID:
                        snprintf(buf, 3, "2\6");
                        if ((write(connfd, buf, strlen(buf))) != strlen(buf)) {
                            if ( isdaemon ) {
                                D_ERR_QUIT ("write in cancel uid error");
                            }
                            else {
                                ERR_QUIT ("write in cancel uid error");
                            }
                        }
                        break;
                    default:                                 /* unreachable */
                        if ( isdaemon ) {
                            D_ERR_QUIT("option");
                        }
                        else {
                            ERR_QUIT("option");
                        }
                        break;
                }
                
                close(fd);
                for (i=0; i < argc; i++) {
                    free(argv[i]);
                }
            }
        }
        close (connfd);
        pthread_cond_signal(&EC);                 /* wakeup thread waiting */
    }
}
