#include "config.h"
#include "resoff.h"

/*======================================================================
  Function req_cancel
  Input : socket file descriptor, arguments of main function
          returned string code from 'resoffd'
  Output : request string for cancel
  requesting to cancel a desk on a time slot
======================================================================*/
void req_cancel (int sockfd, int argc, char **argv)
{
    char sendline[30];           /* send string */
    char recvline[3];            /* received string */
    uid_t uid;                   /* user id */
    uid_t euid;                  /* effective user id */
    uid_t target_uid;            /* target user id */
    struct passwd *puid;         /* passwd information */
    int n;                       /* number of character bytes */
        
    Signal(SIGALRM, sig_alrm);   /* signal set for time out */

    uid = getuid();
    euid = geteuid();

    if ( argc == 4 ) {          /* modules for getting target id and passwd */
        target_uid = uid;
        puid = getpwuid(target_uid);
    }
    else {                             /* argc == 5 */
        puid = getpwnam(argv[4]);
        if ( puid == NULL )
            error_quit('2');
        target_uid = puid->pw_uid;
    }
    
    if ( uid != euid && target_uid != uid ) {   /* no permission to cancel */
        error_quit('4');
    }
    
    n = sizeof(sendline);
    snprintf(sendline, n, "d %d %s %s %s",
             (int)target_uid, argv[2], argv[3], puid->pw_name);
        
    if ( (write(sockfd, sendline, n)) != n )
        ERR_QUIT ("writing");

    alarm(MAXWAIT);             /* wait responds up to MAXWAIT seconds */

    if ( (n = readline(sockfd, recvline, 4)) > 0) {
        recvline[n] = 0;	/* null terminate */
        if ( !(cancel_msg_chk(recvline)) ) 
            cancel_result(recvline);
    }
    else {
        alarm(0);
    }
    
    if ( n < 0 ) {
        error_quit ('9');
    }
}


/*======================================================================
  Function cancel_msg_chk
  Input : received string
  Output : error message or normal return
  cancel message check
======================================================================*/
int cancel_msg_chk (char *recvline)
{
    if ( (recvline[0] >= '0' && recvline[0] <= '9') ||
         (recvline[0] >= 'A' && recvline[0] <= 'I') )
        return (0);

    error_quit('B');
    return (ERROR);                             /* unreachable */
}


/*======================================================================
  Function cancel_result
  Input : received string
  Output : OK or error message
  cancel code interpretation from the server
======================================================================*/
void cancel_result(char *recvline)
{
    switch(recvline[0]) {
        case '0':
            printf("OK: DELETED\n");
            break;
        default:
            error_quit(recvline[0]);
            break;
    }
}
