#include "config.h"
#include "resoff.h"

/*======================================================================
  Function req_reserve
  Input : socket file descriptor, arguments of main function
          returned string code from 'resoffd'
  Output : request string for reservation
  requesting to reserve a desk on a time slot
======================================================================*/
void req_reserve (int sockfd, char **argv)
{
    char sendline[30];          /* send string */
    char recvline[10];          /* receive string */
    uid_t uid;                  /* user id */
    int n;                      /* number of character bytes */
        
    uid = getuid();
    Signal(SIGALRM, sig_alrm);  /* signal set for time out */
    
    n = sizeof(sendline);
    snprintf(sendline, n, "r %d %s %s %s",
             (int)uid, argv[2], argv[3], argv[4]);
    
    if ( (write(sockfd, sendline, n)) != n )
        ERR_QUIT ("writing");

    alarm(MAXWAIT);             /* wait up to MAXWAIT seconds */
    
    if ( (n = readline(sockfd, recvline, 10)) > 0) {
        recvline[n] = 0;	/* null terminate */
        if ( !(rsv_msg_chk(recvline)) ) 
            rsv_result(recvline);
    }
    else {
        alarm(0);
    }

    if ( n < 0 ) {
        error_quit ('9');
    }
    
}


/*======================================================================
  Function rsv_msg_chk
  Input : received string
  Output : error message or normal return
  reservation message check
======================================================================*/
int rsv_msg_chk (char *recvline)
{
    switch (recvline[0]) {
        case '0':
            if ( (recvline[1] >= '0' && recvline[1] <= '9') ||
                 (recvline[1] >= 'A' && recvline[1] <= 'I') )
                return (0);
            else
                error_quit('B');
            break;
        case '1':
        case '2':
        case '3':
        case '4':
            return (NORMAL);
            break;
        default:
            error_quit('B');
            break;
    }

    return (ERROR);                             /* unreachable */
}


/*======================================================================
  Function req_result
  Input : received string
  Output : OK or error message
  result code interpretation from the server
======================================================================*/
void rsv_result(char *recvline)
{
    switch(recvline[0]) {
        case '0':
            error_quit(recvline[1]);
            break;
        case '1':
        case '2':
        case '3':
        case '4':
            printf("OK %c\n", recvline[0]);
            break;
        default:
            ERR_QUIT("unreachable code at rsv_result");
            break;
    }
}
