/*
 File: signals.c
 signal functions for resoff through TCP/IP
 no commented functions from the text, "Unix network programming"
*/

#include "config.h"
#include "resoff.h"

Sigfunc *
signal(int signo, Sigfunc *func)
{
    struct sigaction        act, oact;
    
    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (signo == SIGALRM) {
#ifdef  SA_INTERRUPT
        act.sa_flags |= SA_INTERRUPT;   /* SunOS 4.x */
#endif
    } else {
#ifdef  SA_RESTART
        act.sa_flags |= SA_RESTART;     /* SVR4, 44BSD */
#endif
    }
    if (sigaction(signo, &act, &oact) < 0)
        return(SIG_ERR);
    return(oact.sa_handler);
}

Sigfunc *
Signal(int signo, Sigfunc *func)        /* for our signal() function */
{
    Sigfunc *sigfunc;
    
    if ( (sigfunc = signal(signo, func)) == SIG_ERR)
        ERR_QUIT("signal error");
    return(sigfunc);
}

void
sig_chld(int signo)
{
    pid_t	pid;
    int		stat;

    while ( (pid = waitpid(-1, &stat, WNOHANG)) > 0);

    return;
}

/*======================================================================
  Function sig_alrm
  Input : signal number
  Output : calling error message printing and quit function
  timeout signal handling function - error message and quit
======================================================================*/
void sig_alrm(int signo)
{
    error_quit('A');
}

/*======================================================================
  Function sig_int
  Input : signal number
  Output : calling error message printing and quit function
  SIGINT signal handling function - remove lock file, printing error
                                    message, and quit
======================================================================*/
void sig_int(int signo)
{
    extern char lockfile[256];        /* lock file name */
    
    unlink (lockfile);
    write(2, "resoffd: terminated by signal SIGINT\n", 29);
    _exit(-1);
}
