#include "config.h"
#include "resoff.h"

/*======================================================================
  Function openerrmsg
  Input : 
  Output : error message
  print open error message
======================================================================*/
void openerrmsg() 
{
    extern int isdaemon;
    
    if ( isdaemon ) {
        d_openerrmsg();
        return;
    }
    
    switch(errno) {
        case EACCES:
            fprintf(stderr, "The requested access to the file is not "
                    "allowed.\n");
            break;
        case ENOENT:
            fprintf(stderr, "The subdirectory does not exist.\n");
            break;
        case ENOTDIR:
            fprintf(stderr, "Given directory is not a directory.\n");
            break;
        case ENOSPC:
            fprintf(stderr, "File created but no room for the new file.\n");
            break;
        case ENFILE:
            fprintf(stderr, "The  limit on the total number of files open "
                    "on the system has been reached.\n");
            break;
        default:
            perror("open error");
            break;
    }
}

/*======================================================================
  Function d_openerrmsg
  Input : 
  Output : syslog error message
  print open error message to syslog
======================================================================*/
void d_openerrmsg() 
{
    switch(errno) {
        case EACCES:
            syslog(LOG_ERR, "The requested access to the file is not "
                    "allowed.");
            break;
        case ENOENT:
            syslog(LOG_ERR, "The subdirectory does not exist.");
            break;
        case ENOTDIR:
            syslog(LOG_ERR, "Given directory is not a directory.");
            break;
        case ENOSPC:
            syslog(LOG_ERR, "File created but no room for the new file.");
            break;
        case ENFILE:
            syslog(LOG_ERR, "The  limit on the total number of files open "
                    "on the system has been reached.");
            break;
        default:
            syslog(LOG_ERR, strerror(errno));
            break;
    }
}

/*======================================================================
  Function socketerrmsg
  Input : 
  Output : error message
  print socket error message
======================================================================*/
void socketerrmsg() 
{
    extern int isdaemon;
    
    if ( isdaemon ) {
        d_socketerrmsg();
        return;
    }

    switch(errno) {
        case EPROTONOSUPPORT:
        case EINVAL:
            fprintf(stderr, "Protocol error.\n");
            break;
        case ENFILE:
        case ENOBUFS:
        case ENOMEM:
            fprintf(stderr, "Not enough memory for socket.\n");
            break;
        case EACCES:
            fprintf(stderr, "No permission to create a socket.\n");
            break;
        default:
            perror("socket error");
            break;
    }
}

/*======================================================================
  Function d_socketerrmsg
  Input : 
  Output : error message to syslog
  print socket error message to syslog
======================================================================*/
void d_socketerrmsg() 
{
    switch(errno) {
        case EPROTONOSUPPORT:
        case EINVAL:
            syslog(LOG_ERR, "Protocol error.");
            break;
        case ENFILE:
        case ENOBUFS:
        case ENOMEM:
            syslog(LOG_ERR, "Not enough memory for socket.");
            break;
        case EACCES:
            syslog(LOG_ERR, "No permission to create a socket.");
            break;
        default:
            syslog(LOG_ERR, strerror(errno));
            break;
    }
}

/*======================================================================
  Function binderrmsg
  Input : 
  Output : error message
  print bind error message
======================================================================*/
void binderrmsg()
{
    extern int isdaemon;
    
    if ( isdaemon ) {
        d_binderrmsg();
        return;
    }

    switch(errno) {
        case EBADF:
        case ENOTSOCK:
        case ENAMETOOLONG:
            fprintf(stderr, "Invalid parameter.\n");
            break;
        case EINVAL:
            fprintf(stderr, "The socket is already bound to  an  address.\n");
            break;
        case EACCES:
            fprintf(stderr, "The address is protected, and the user is  not "
                    "the super-user.\n");
            break;
        case EFAULT:
            fprintf(stderr, "The address is outside of the user's accessible "
                    "address space.\n");
            break;
        case ENOMEM:
            fprintf(stderr, "Insufficient kernel memory.\n");
            break;
        default:
            perror("bind error");
            break;
    }
}

/*======================================================================
  Function d_binderrmsg
  Input : 
  Output : error message to syslog
  print bind error message to syslog
======================================================================*/
void d_binderrmsg()
{
    switch(errno) {
        case EBADF:
        case ENOTSOCK:
        case ENAMETOOLONG:
            syslog(LOG_USER, "Invalid parameter.\n");
            break;
        case EINVAL:
            syslog(LOG_USER, "The socket is already bound to  an  address.\n");
            break;
        case EACCES:
            syslog(LOG_USER, "The address is protected, and the user is  not "
                    "the super-user.\n");
            break;
        case EFAULT:
            syslog(LOG_USER, "The address is outside of the user's accessible "
                    "address space.\n");
            break;
        case ENOMEM:
            syslog(LOG_USER, "Insufficient kernel memory.\n");
            break;
        default:
            syslog(LOG_USER, strerror(errno));
            break;
    }
}

/*======================================================================
  Function connerrmsg
  Input : 
  Output : error message
  print connect error message
======================================================================*/
void connerrmsg()
{
    switch(errno) {
        case EFAULT:
            error_quit('6');
            break;
        default:
            /*perror("connection error");*/ /* remain for debugging */
            error_quit('7');
            break;
    }
}
