#include "config.h"
#include "resoff.h"

/*======================================================================
  Function server_cmdchk
  Input : received string, pseudo arguments, user id
  Output : option number of reserve, cancel or list
           Return value:
	     -r option : _RESERVE (1)
	     -d option : _CANCEL (2)
	     -l option : _LIST (3)
             reserve command error : _ERR_RESERVE (11)
             cancel command error : _ERR_CANCEL (12)
             client syntax error : _ERR_SYNTAX (20)
             reserve syntax error : _ERR_RSV_SYNTAX (21)
             cancel syntax error : _ERR_CNL_SYNTAX (22)
             reserve uid error : _ERR_RSV_UID (31)
             cancel uid error : _ERR_CNL_UID (32)
	     otherwise : 0
  determine option number from request string
  pseudo arguments used for portability of sharing functions with client
  and reusing functions of assignment 2
======================================================================*/
int server_cmdchk (char *recvline, int *argc, char **argv, uid_t *uid)
{
    int i;
    char *argvs[5];             /* received server part arguments */
    char *p;                    /* temporary pointer for string allocation */
    char word[10];              /* a word string */
    char n;                     /* length of the word */
    struct passwd *puid;        /* passwd for user name and uid */
    char *ptr;                  /* pointer to some of recvline */
    int count = 0;              /* temporary pointer to recvline */
    int len;                    /* length of recvline */
    
    *argc = 0;
    len = strlen(recvline);
 
    /* dummy argument for the old form of the main arguments */
    p = malloc(8);
    strcpy(p, "resoffd");
    argv[0] = p;

    /* read the received arguments as words */
    ptr = &recvline[0];
    for (i=0; i<6; i++) {
        if ( count >= len ) break;
        if ( (n = readword(ptr, word, 10)) > 0 ) {
            p = malloc(n);
            strcpy(p, word);
            argvs[i] = p;
            count += n;
            ptr = &recvline[++count];
        }
        else break;

        if ( i == 5 ) {               /* too many arguments */
            return (_ERR_SYNTAX);
        }
    }

    if ( i == 0 )
        return (_ERR_SYNTAX);
    
    if ( i > 1 )
        *argc = i;                             /* r or d */
    else
        *argc = 2;                             /* l */

    argv[1] = argvs[0];                        /* argvs[1] is userid */
    
    for (i=2; i<*argc; i++) {
        argv[i] = argvs[i];
    }
    

    if ( *argc > 2 ) {
        if ( is_all_digit(argvs[1]) )
            puid = getpwuid( (*uid = (uid_t)atoi(argvs[1])) );
        else 
            return (_ERR_SYNTAX);
    }
    
    if ( puid == NULL && *argc > 2 )
        return (FALSE);
        
    if ( strlen(argv[1]) == 1 ) {              /* r, d, l */
        switch( argv[1][0] ) {
            case 'r' :
                if ( *argc == 5 ) {
                    if ( puid == NULL )        /* invalid userid in reserve */
                        return (_ERR_RSV_UID);
                    if ( is_rsv_cmd(argv) )
                        return (_RESERVE);
                    else
                        return (_ERR_RSV_SYNTAX);
                }
                else return (_ERR_RESERVE);
                break;
            case 'd' :
                if ( *argc == 5 ) { 
                    if ( puid == NULL )        /* invalid userid in cancel */
                        return (_ERR_CNL_UID);
                    if ( is_cancel_cmd(argv) )
                        return (_CANCEL);
                    else
                        return (_ERR_CNL_SYNTAX);
                }
                else return (_ERR_CANCEL);
                break;
            case 'l' :
                if ( *argc == 2 ) { return (_LIST); }
                break;
            default:
                return (FALSE);
	}
    }
    return (FALSE);
}

