#include "config.h"
#include "resoff.h"
#include "mask.h"

/*======================================================================
  Function validity
  Input : database index, a time block data, course name
  Output : error message or return normally
  check validity of given options - cannot reserve more than 3 a day
                                    cannot reserve the same course
======================================================================*/
char validity (int db_index, Room slot, char *course, int fd, uid_t uid) {

    int i;

    if ( slot.count != 0 ) {         /* if block is not empty */
        for (i=0; i<4; i++) {
	    if ( !slot.empty[i] ) {  /* if empty desk */
	        if ( slot.desk[i].uid == uid ) {   /* the same user id */
                    return('E');     /* currently none exist error code */
		}                              /* the same course name */
		else if ( !strcmp(slot.desk[i].class, course) ) {
		    /* conflict course reservation */
		    return('I');
		}
	    }
	}
    }

    /* more than three reservation a day? */
    return ( (check_day_reserve (db_index, fd, uid)) );
  
}

/*======================================================================
  Function check_day_reserve
  Input : database index
  Output : error message or return normally
  check validity of given options - cannot reserve more than 3 a day
======================================================================*/
char check_day_reserve (int db_index, int fd, uid_t uid) {

    int rcount = 0;                 /* resevation number of a user */
    int i, j;                       /* loop index */
    int begin, max_slot;            /* begin index, maximum block of a day */
    int loc;                        /* location of a block */
    int len;                        /* number of bytes read */
    int day;                        /* day index */
    sigset_t newset, oldset;        /* new and old signal sets */
    extern int isdaemon;
    extern pthread_mutex_t IOM;     /* I/O mutex variable */
    
    Room slot;                      /* a time block */
    
    day = get_day (db_index);       /* calculate a day from index */
    if ( day < 0 ) {                /* unreachable value */
        if ( isdaemon ) {
            D_ERR_QUIT("db_index error");
        }
        else {
            ERR_QUIT("db_index error");
        }
    }

    switch (day) {
    case 0:                         /* Monday */
        begin = 0;
	max_slot = 11;
	break;
    case 1:                         /* Wednesday */
        begin = 11;
	max_slot = 11;
	break;
    case 2:                         /* Friday */
        begin = 2 * 11;
	max_slot = 11;
	break;
    case 3:                         /* Tuesday */
        begin = 33;
	max_slot = 9;
	break;
    case 4:                         /* Thursday */
        begin = 33 + 9;
	max_slot = 9;
	break;
    case 5:                         /* Saturday */
        begin = 33 + 9 * 2;
	max_slot = 9;
	break;
    case 6:                         /* Sunday */
        begin = 33 + 9 * 3;
	max_slot = 9;
	break;
    default:
        ERR_QUIT("illigal db_index error");
	break;
    }

    SIGFILL(newset);

    /* counting the number of reservaion of a user on a day */
    for ( i = begin; i < begin + max_slot; i++ ) {
        loc = i * sizeof (slot);                    /* block location */
	BLOCK(newset, oldset);
        pthread_mutex_lock(&IOM);
	CHECK (lseek (fd, loc, SEEK_SET));
	CHECK (len = read (fd, &slot, sizeof(slot)));
        pthread_mutex_unlock(&IOM);
	UNBLOCK(newset, oldset);
	if ( slot.count > 0 ) {
	    for (j=0; j<4; j++) {
	        if ( !slot.empty[j] ) {         /* if desk is not empty */
		    if ( slot.desk[j].uid == uid ) {
		        rcount++;
		    }
		}
	    }
	}
        
	if ( rcount > 2 ) {
            /* you cannot reserver more than three slots a day */
            return('F');
	}
    }
    return (NORMAL);
}
